import { Component, ElementRef, ViewChild, OnInit, Input, OnDestroy, HostListener } from '@angular/core';

import {
  Label,
  MySprite, ShapeRect, tweenChange,

} from './Unit';
import { res, resAudio } from './resources';

import { Subject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';

import TWEEN from '@tweenjs/tween.js';
import { getDefaultTile } from '../bean/TitleBean';
import { getDefaultExercises, getDefaultWord } from '../bean/ExercisesBean';
import { Listening } from './Listening';
import { Title } from './Title';
import { EexercisesA, EexercisesB } from './Exercises';
import { typesetting } from './TypesettingUtil';
import { defaultVal } from './defaultValue.js';



@Component({
  selector: 'app-play',
  templateUrl: './play.component.html',
  styleUrls: ['./play.component.css']
})
export class PlayComponent implements OnInit, OnDestroy {

  @ViewChild('canvas', { static: true }) canvas: ElementRef;
  @ViewChild('wrap', { static: true }) wrap: ElementRef;

  // 数据
  data;

  ctx;

  canvasWidth = 1280; // canvas实际宽度
  canvasHeight = 720; // canvas实际高度

  canvasBaseW = 1280; // canvas 资源预设宽度
  canvasBaseH = 720;  // canvas 资源预设高度

  mx; // 点击x坐标
  my; // 点击y坐标


  // 资源
  rawImages = new Map(res);
  rawAudios = new Map(resAudio);

  images = new Map();

  animationId: any;
  winResizeEventStream = new Subject();

  audioObj = {};

  renderArr;
  mapScale = 1;

  canvasLeft;
  canvasTop;

  saveKey = 'YM5-23';

  canTouch = true;

  /**标题*/
  title;

  listening;

  curAudio;

  exercisesArr = [];


  @HostListener('window:resize', ['$event'])
  onResize(event) {
    this.winResizeEventStream.next();
  }


  ngOnInit() {

    this.data = {};

    // 获取数据
    const getData = (<any>window).courseware.getData;
    getData((data) => {

      if (data && typeof data == 'object') {
        this.data = data;
      }
      else{
        this.data = defaultVal;
      }
      //console.log('data:',JSON.stringify(data) );

      // 初始化 各事件监听
      this.initListener();

      // 若无数据 则为预览模式 需要填充一些默认数据用来显示
      this.initDefaultData();

      // 初始化 音频资源
      this.initAudio();
      // 初始化 图片资源
      this.initImg();
      // 开始预加载资源
      this.load();

    }, this.saveKey);

  }

  ngOnDestroy() {
    window['curCtx'] = null;
    window.cancelAnimationFrame(this.animationId);
    this.cleanAudio();
  }


  cleanAudio() {
    if (this.audioObj) {
      for (const key in this.audioObj) {
        this.audioObj[key].pause();
      }
    }
  }


  load() {

    // 预加载资源
    this.loadResources().then(() => {
      window["air"].hideAirClassLoading(this.saveKey, this.data);
      this.init();
      this.update();
    });
  }


  init() {

    this.initCtx();
    this.initData();
    this.initView();
  }

  initCtx() {
    this.canvasWidth = this.wrap.nativeElement.clientWidth;
    this.canvasHeight = this.wrap.nativeElement.clientHeight;
    this.canvas.nativeElement.width = this.wrap.nativeElement.clientWidth;
    this.canvas.nativeElement.height = this.wrap.nativeElement.clientHeight;


    this.ctx = this.canvas.nativeElement.getContext('2d');
    this.canvas.nativeElement.width = this.canvasWidth;
    this.canvas.nativeElement.height = this.canvasHeight;

    window['curCtx'] = this.ctx;
    window['curImages'] = this.images;
  }






  updateItem(item) {
    if (item) {
      item.update();
    }
  }

  updateArr(arr) {
    if (!arr) {
      return;
    }
    for (let i = 0; i < arr.length; i++) {
      arr[i].update(this);
    }
  }







  initListener() {

    this.winResizeEventStream
      .pipe(debounceTime(500))
      .subscribe(data => {
        this.renderAfterResize();
      });


    // ---------------------------------------------
    const setParentOffset = () => {
      const rect = this.canvas.nativeElement.getBoundingClientRect();
      this.canvasLeft = rect.left;
      this.canvasTop = rect.top;
    };
    const setMxMyByTouch = (event) => {
      if (event.touches.length <= 0) {
        return;
      }
      if (this.canvasLeft == null) {
        setParentOffset();
      }
      this.mx = event.touches[0].pageX - this.canvasLeft;
      this.my = event.touches[0].pageY - this.canvasTop;
    };

    const setMxMyByMouse = (event) => {
      this.mx = event.offsetX;
      this.my = event.offsetY;
    };
    // ---------------------------------------------


    let firstTouch = true;

    const touchDownFunc = (e) => {
      if (firstTouch) {
        firstTouch = false;
        removeMouseListener();
      }
      setMxMyByTouch(e);
      this.mapDown(e);
    };
    const touchMoveFunc = (e) => {
      setMxMyByTouch(e);
      this.mapMove(e);
    };
    const touchUpFunc = (e) => {
      setMxMyByTouch(e);
      this.mapUp(e);
    };

    const mouseDownFunc = (e) => {
      if (firstTouch) {
        firstTouch = false;
        removeTouchListener();
      }
      setMxMyByMouse(e);
      this.mapDown(e);
    };
    const mouseMoveFunc = (e) => {
      setMxMyByMouse(e);
      this.mapMove(e);
    };
    const mouseUpFunc = (e) => {
      setMxMyByMouse(e);
      this.mapUp(e);
    };


    const element = this.canvas.nativeElement;

    const addTouchListener = () => {
      element.addEventListener('touchstart', touchDownFunc);
      element.addEventListener('touchmove', touchMoveFunc);
      element.addEventListener('touchend', touchUpFunc);
      element.addEventListener('touchcancel', touchUpFunc);
    };
    const removeTouchListener = () => {
      element.removeEventListener('touchstart', touchDownFunc);
      element.removeEventListener('touchmove', touchMoveFunc);
      element.removeEventListener('touchend', touchUpFunc);
      element.removeEventListener('touchcancel', touchUpFunc);
    };

    const addMouseListener = () => {
      element.addEventListener('mousedown', mouseDownFunc);
      element.addEventListener('mousemove', mouseMoveFunc);
      element.addEventListener('mouseup', mouseUpFunc);
    };
    const removeMouseListener = () => {
      element.removeEventListener('mousedown', mouseDownFunc);
      element.removeEventListener('mousemove', mouseMoveFunc);
      element.removeEventListener('mouseup', mouseUpFunc);
    };

    addMouseListener();
    addTouchListener();
  }


  playAudio(key, now = false, callback = null) {

    const audio = this.audioObj[key];
    if (audio) {

      this.pauseAudio(this.curAudio);

      if (now) {
        audio.pause();
        audio.currentTime = 0;
      }

      if (callback) {
        audio.onended = () => {
          callback();
        };
      }
      audio.play();

      this.curAudio = audio;
    }

    return audio;
  }

  pauseAudio(audio) {
    if (audio) {
      if (this.listening && audio == this.listening.audio) {
        this.listening.pauseAudio();
      }
      else {
        audio.pause();
      }
    }
  }



  loadResources() {
    const pr = [];
    this.rawImages.forEach((value, key) => {// 预加载图片

      const p = this.preload(value)
        .then(img => {
          this.images.set(key, img);
        })
        .catch(err => console.log(err));

      pr.push(p);
    });

    this.rawAudios.forEach((value, key) => {// 预加载音频

      const a = this.preloadAudio(value)
        .then(() => {
          // this.images.set(key, img);
        })
        .catch(err => console.log(err));

      pr.push(a);
    });
    return Promise.all(pr);
  }

  preload(url) {
    return new Promise((resolve, reject) => {
      const img = new Image();
      // img.crossOrigin = "anonymous";
      img.onload = () => resolve(img);
      img.onerror = reject;
      img.src = url;
    });
  }

  preloadAudio(url) {
    return new Promise((resolve, reject) => {
      const audio = new Audio();
      audio.oncanplay = (a) => {
        resolve(a);
      };
      audio.onerror = () => {
        reject();
      };
      audio.src = url;
      audio.load();
    });
  }


  renderAfterResize() {
    this.canvasWidth = this.wrap.nativeElement.clientWidth;
    this.canvasHeight = this.wrap.nativeElement.clientHeight;
    this.init();
  }





  checkClickTarget(target) {

    const rect = target.getBoundingBox();

    if (this.checkPointInRect(this.mx, this.my, rect)) {
      return true;
    }
    return false;
  }

  getWorlRect(target) {

    let rect = target.getBoundingBox();

    if (target.parent) {

      const pRect = this.getWorlRect(target.parent);
      rect.x += pRect.x;
      rect.y += pRect.y;
    }
    return rect;
  }

  checkPointInRect(x, y, rect) {
    if (x >= rect.x && x <= rect.x + rect.width) {
      if (y >= rect.y && y <= rect.y + rect.height) {
        return true;
      }
    }
    return false;
  }





  addUrlToAudioObj(key, url = null, vlomue = 1, loop = false, callback = null) {

    const audioObj = this.audioObj;

    if (url == null) {
      url = key;
    }

    this.rawAudios.set(key, url);

    const audio = new Audio();
    audio.src = url;
    audio.load();
    audio.loop = loop;
    audio.volume = vlomue;

    audioObj[key] = audio;

    if (callback) {
      audio.onended = () => {
        callback();
      };
    }
  }

  addUrlToImages(url) {
    this.rawImages.set(url, url);
  }






  // ======================================================编写区域==========================================================================





  /**
   * 添加默认数据 便于无数据时的展示
   */
  initDefaultData() {

    if (!this.data.title) {
      this.data.title = getDefaultTile();
    }

    if (!this.data.exercisesArr) {
      this.data.exercisesArr = [];
    }

    for (let i = this.data.exercisesArr.length; i < 5; ++i) {

      let exercises = getDefaultExercises();

      this.data.exercisesArr.push(exercises);

    }

    for (let i = 0; i < this.data.exercisesArr.length; ++i) {

      let exercises = this.data.exercisesArr[i];

      if (!exercises.wordArr) {
        exercises.wordArr = [];
      }

      let len = exercises.wordArr.length;

      for (let j = len; j < 3; ++j) {

        let word = getDefaultWord();
        exercises.wordArr.push(word);
      }
    }
  }


  /**
   * 添加预加载图片
   */
  initImg() {



  }

  /**
   * 添加预加载音频
   */
  initAudio() {

    // 音效
    this.rawAudios.forEach((item, key) => {
      // 音效
      this.addUrlToAudioObj(key, item);
    })

    //标题发音
    if (this.data.title.audioUrl) {
      this.addUrlToAudioObj('titleAudio', this.data.title.audioUrl);
    }


    //听力材料
    if (this.data.listenAudioUrl) {
      this.addUrlToAudioObj('listenAudio', this.data.listenAudioUrl);
    }

    for (let i = 0; i < this.data.exercisesArr.length; ++i) {

      let exercises = this.data.exercisesArr[i];

      if (exercises.audioUrl) {
        this.addUrlToAudioObj(exercises.audioUrl);
      }

      for (let j = 0; j < exercises.wordArr.length; ++j) {
        let word = exercises.wordArr[j];

        if (word.audioUrl) {
          this.addUrlToAudioObj(word.audioUrl);
        }
      }


    }

  }



  /**
   * 初始化数据
   */
  initData() {

    const sx = this.canvasWidth / this.canvasBaseW;
    const sy = this.canvasHeight / this.canvasBaseH;
    const s = Math.min(sx, sy);
    this.mapScale = s;


    this.renderArr = [];

    this.exercisesArr = [];

  }



  /**
   * 初始化试图
   */
  initView() {

    this.initBg();


    this.initTitle();

    this.initBottomPart();

    this.initExercises();
  }

  initBg() {

    const bg = new ShapeRect();

    bg.setSize(this.canvasWidth, this.canvasHeight);
    bg.x = 0;
    bg.y = 0;
    bg.fillColor = this.data.skin == 'A' ? '#f7effc' : '#fcf8e9';

    this.renderArr.push(bg);
  }

  initBottomPart() {

    if (!this.data.listenAudioUrl) {
      return;
    }

    const listening = new Listening(this.audioObj['listenAudio'], this.images);
    listening.init(this.mapScale, this.mapScale);

    listening.bg.x = this.canvasWidth - listening.bg.width - 10 * this.mapScale;
    listening.bg.y = this.canvasHeight - listening.bg.height - 6.5 * this.mapScale;
    this.listening = listening;

    this.renderArr.push(listening.bg);
  }

  initTitle() {
    const title = new Title(this.data.title, this.images);

    title.init(this.mapScale, this.mapScale);
    title.audio = this.audioObj['titleAudio'];
    this.title = title;

    title.titleBg.y = 0;
    title.titleBg.x = 0;

    this.renderArr.push(title.titleBg);
  }


  initExercises() {

    let h = 0;

    let bgArr = [];

    let maxTitleScale = this.mapScale;

    let wordScale = this.mapScale;

    for (let i = 0; i < this.data.exercisesArr.length; ++i) {
      let data = this.data.exercisesArr[i];

      let exercises;

      if (this.data.skin === 'A') {
        exercises = new EexercisesA();
        exercises.titleColor = "#5f1782";
      }
      else {
        exercises = new EexercisesB();
        exercises.titleColor = "#875f00";
      }

      exercises.init(data, this.images, this.mapScale, this.mapScale);

      if(maxTitleScale > exercises.title.scaleX){
        maxTitleScale = exercises.title.scaleX;
      }

      if(wordScale > exercises.wordBg.scaleX){
        wordScale = exercises.wordBg.scaleX;
      }

      h = exercises.bg.height;

      this.exercisesArr.push(exercises);

      this.renderArr.push(exercises.bg);

      bgArr.push(exercises.bg);
    }

    //重新设置字体大小
    for(let i = 0; i < this.exercisesArr.length; ++ i){
      this.exercisesArr[i].resetScale(maxTitleScale, wordScale);
    }

    //进行排版操作
    typesetting(bgArr, 3, this.canvasWidth, this.canvasHeight, h, 14 * this.mapScale, 78 * this.mapScale);
  }





  mapDown(event) {

    if (!this.canTouch) {
      return;
    }

    if (this.listening && this.checkClickTarget(this.listening.bg)) {

      let tempAudio = this.curAudio;

      if (tempAudio) {
        tempAudio.pause();
      }

      this.curAudio = this.listening.play(() => {
        this.listening.reset();
      });

      return;
    }

    if (this.checkClickTarget(this.title.titleBg)) {
      this.pauseAudio(this.curAudio);
      let audio = this.title.playAudio();
      if (audio) {
        this.curAudio = audio;
      }
      return;
    }

    let clicked = false;

    let target;

    let callback;

    for (let i = 0; i < this.exercisesArr.length; ++i) {

      let exer = this.exercisesArr[i];

      if (this.checkClickTarget(exer.bg)) {
        callback = this.clickExercises.bind(this);
        clicked = true;
        target = exer;
        break;
      }
    }

    if (clicked) {
      callback(target);
      return;
    }
  }

  mapMove(event) {

  }

  mapUp(event) {

  }



  update() {

    // ----------------------------------------------------------
    this.animationId = window.requestAnimationFrame(this.update.bind(this));
    // 清除画布内容
    this.ctx.clearRect(0, 0, this.canvasWidth, this.canvasHeight);
    // tween 更新动画
    TWEEN.update();
    // ----------------------------------------------------------



    this.updateArr(this.renderArr);


  }



  clickExercises(item) {

    if (this.checkClickTarget(item.titleBg)) {
      this.playAudio(item.data.audioUrl, true);
      return;
    }

    for (let i = 0; i < item.wordArr.length; ++i) {

        let word = item.wordArr[i];

        if(this.checkClickTarget(word.textBg)){

          if(word.data.isFill == '1'){

            this.playAudio('accompany', true);
            if(word.alpha == 0){
              word.alpha = 1;
              word.underLine.alpha = 0;
            }
            else{
              this.playAudio(word.data.audioUrl, true);
            }
          }
          else{
            this.playAudio(word.data.audioUrl, true);
          }
          break;
        }
    }
  }
}
